/*
 * main.h
 *
 * Created 14 Nov 2013
 * Author: Joel Heck
 * 
 * Adapted from code "main.h" created by Brian on 30 Mar 2012 for a 4760
 * project
 * 
 * Defines data structures and declares functions used for navigation and data
 * storage
 */

#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef MAIN_H_
#define MAIN_H_

#define TRUE 1
#define FALSE 0
#define F_CPU 16000000UL

// Constants used for navigation functions
#define ERADIUS 6371000	// Approximate Earth radius in meters
#define DEGRAD 0.017453	// Conversion from degrees to radians (pi/180)

//typedef for interrupt callback functions.  watch out for non-volatile variables!
typedef void (*int_func)(char);

// Position object to store GPS data
typedef struct {
	
    uint32_t gps_time;      // Current GPS time in seconds
    uint32_t elapsed_time;  // Elapsed time of run in seconds
    uint8_t hour;           // Local hour time
    uint8_t minute;         // Local minute time

    int8_t speed;           // Speed in MPH (from GPS)
    int16_t heading;        // True course in degrees (from GPS)
                            // Only good if speed >4 MPH

    uint16_t dist_rem;      // Distance to next waypt (meters)
    double dist_done;        // Current completed distance (kilometers)

    double latitude;         // Current latitude in degrees
    double longitude;        // Current longitude in degrees
    int16_t altitude;       // Current altitude in meters

    double prev_lat;         // Current latitude in degrees
    double prev_lon;        // Current longitude in degrees
} Position;

// Navigation functions
uint16_t approx_dist_to_waypt(double, double, double, double);
int16_t bearing_to_waypt(double, double, double, double);
int8_t dir_to_waypt(int16_t, int16_t, uint8_t);

// Time formatting function for LCD display
void print_time(uint8_t);

#endif /* MAIN_H_ */
