/****************************************************
*													*
*	scheduler.c 									*
*													*
*	TIMER0 interrupt for 1ms timebase				*
*	scheduler determines program state				*
*													*
****************************************************/

#include <stdio.h> 
#include "scheduler.h"
#include "tasks.h"
#include "fatfs/diskio.h"


/* Program states */
// menu, stats, navigation
#define MENU 1
#define STATS 2
#define ROUTE_SELECT 3
#define NAVIGATION 4
#define LOGGER 5

/* State parameters */
uint8_t program_state;
volatile uint8_t menu_clock;
volatile uint16_t nav_clock;
volatile char diskTime;

/* Task timers */
uint8_t task_time = 3;
uint8_t LCD_time = 20;
#define MENU_TIME 3  //30 ms to reset task_time
#define NAV_TIME 100 //1 sec to reset task_time in NAVIGATION state


/* TIMER0 comare match ISR */
ISR (TIMER0_COMPA_vect)  
{ 
	if(menu_clock>0) menu_clock--;
	if(nav_clock>0) nav_clock--;
	//SD Card Timeout clock (Must be called every 10ms)
	if (diskTime>0) --diskTime;
    if (diskTime == 0) {
			diskTime = 10; //ms
            disk_timerproc();
    }
} 

/* Scheduler initialization (called once) */
void init_scheduler(void){
	program_state = MENU;				// Initial state main menu
	menu_clock = MENU_PERIOD;	// Start scheduler
	nav_clock = NAV_PERIOD;		// Called every 1 second
}

/* Scheduler loop state machine */
void scheduler(void){
	uint8_t status;
	if(nav_clock ==0){
		nav_clock = NAV_PERIOD;
		if(program_state==NAVIGATION){
			navigation();
		} else if (program_state==LOGGER){
			log_waypt();
		}
	} // END if(nav_clock...
	else if(menu_clock == 0){
		switch (program_state){
			
			/* Main Menu */
			case MENU:
				status = menu(0);
				if (status == 1) { // Goto route select
					route_select(1);
					program_state= ROUTE_SELECT;
				}
				else if (status == 2) { // Goto Route Logger
					if(!logger(1)) program_state= LOGGER;
				}
				else if (status == 3) { // Goto statistics
					stats(1);
					program_state= STATS;
				}
				break;
			
			/* Statistics Menu */
			case STATS:
				status = stats(0);
				if (status == 1){ // Go back
					menu(1);
					program_state= MENU;
				}
				break;
			
			/* Route Selection Menu */
			case ROUTE_SELECT:
				status = route_select(0);
				//program_state= NAVIGATION;
				if (status==1){ // Go back to main menu
					menu(1);
					program_state= MENU;
				}
				else if (status==2){
					program_state= NAVIGATION;
					nav_clock = 0;
				}
				break;
			case NAVIGATION:
				if(nav_button()){
					program_state= MENU;
					PORTA &= ~(0x01<<4);
					PORTA &= ~(0x01<<5);
					menu(1);
				}
			
			case LOGGER:		// GPS Logger
				status = logger(0);
				if (status == 1){
					menu(1);
					program_state= MENU;
				}
				break;
		}// END switch (program_state)
		menu_clock = MENU_PERIOD; // Restart scheduler countdown
	}//END else if
}// END scheduler
